#ifdef __APPLE__
#include <OpenGL/gl.h>
#include <GLUT/glut.h>
#else
#include <GL/gl.h>
#include <GL/glut.h>
#endif

#include "main.h"
#include "hier.h"

static int quit = FALSE;
static double mouse_x = 0;
static double mouse_y = 0;

static Scene *scn;

static int w_mouse_x;
static int w_mouse_y;


void get_my_mouse()
{
    Real x, y;
    
    mouse_x = (w_mouse_x/512.) * 6.28;
    mouse_y = (w_mouse_y/512.) * 6.28;
}

void my_xform_cb(double *p, int n, void *info)
{
  int i;
  get_my_mouse();
  int *assgn = info;
  for (i=0; i < n; i++) {
    switch (assgn[i]) {
    case 0: p[i] = mouse_x; break;
    case 1: p[i] = mouse_y; break;
    default: break;
    }
  }
}

Val xmouse_cb_parse(int pass, Pval *pl)
{
  Val v = {V_NULL, {0}};
  if (pass == T_EXEC) { Pval *p;
    Transform *t = NULL;
    int i, *info = NULL;
    double par[9] = {0,0,0, 0,0,0, 1,1,1};
    double assgn[9] = {-1, -1, -1, -1, -1, -1, -1, -1, -1};
    Matrix4 m = m4_ident();
    Matrix4 mi = m4_ident();
    for (p = pl; p != NULL; p = p->next) {
      if (p->name && strcmp(p->name, "assign") == 0 && p->val.type == V_PVL ) {
	pvl_to_array(p->val.u.v, assgn, 9);
	info = emalloc(sizeof(int) * 9);
	for (i=0; i<9; i++)
	  info[i] = assgn[i];
      } else
	error("(xparam) wrong argument");
    }
    t = xform_new(XFORM_PARAM);
    xform_setpar(t, par);
    xform_updatepar(t);
    xform_applypar_m(t);
    xform_applypar_mi(t);
    t->p_cb = my_xform_cb;
    t->info = info;
    v.u.v = t;
    v.type = V_TRANSFORM;
  }
  return v;
}


// -------

void display(void)
{
  display_gl(scn);
  glutSwapBuffers();
  glutPostRedisplay();
  time_incr(1);
}

void process_key(unsigned char key, int x, int y) 
{
  if (key == 27 || key == 'q') 
    exit(0);
}

void idle(void)
{
  glutPostRedisplay();
}

void init_glut(char *name, Scene *s)
{
  int ac = 0; char *av[1] = {""};
  glutInit(&ac, av);
  glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);
  glutInitWindowSize(s->img->w, s->img->h);
  glutCreateWindow(name);

  glutDisplayFunc( display );
  glutKeyboardFunc( process_key );
  glutIdleFunc( idle );

  scn = s;
}

void update_mouse(int x, int y)
{
  w_mouse_x = x;
  w_mouse_y = y;
}

// ----------

int main(int argc, char **argv)
{
  Scene *s, *s1;
  int k; Object *o;

  init_lang();
  s = scene_read();
  init_glut("GPU", s);

  init_gl(s);
  load_objects_gl(s->objs);
  setup_programs_gl(s, s->objs);

  glutPassiveMotionFunc( update_mouse );
  glutMainLoop();

  close_gl(s);
  cleanup_programs_gl();

  exit(0);
}


void init_lang(void)
{
  lang_defun("scene", scene_parse);
  lang_defun("view", view_parse);

  lang_defun("primobj", obj_parse);
  lang_defun("sphere", sphere_parse);

  lang_defun("dist_light", distlight_parse);
  lang_defun("plastic", plastic_parse);

  lang_defun("group", graph_parse);
  lang_defun("affine", xaffine_parse);
  lang_defun("param", xparam_parse);
  lang_defun("mouse", xmouse_cb_parse);

  lang_defun("arg", arg_parse);
  lang_defun("motor", motor_parse);

  lang_defun("include", include_parse);
}

