#include "main.h"

static Scene *s;
static RContext *rc;
static Matrix4 mclip, mdpy;


int main(int argc, char **argv)
{
  Color c; int u, v;
  Ray r;

  init_lang();
  s = scene_read();
  init_render(s);
  s->objs = graph_flatten(graph_transform(s->objs));
 
  for (v = s->view->sc.ll.y; v < s->view->sc.ur.y; v += 1) {
    for (u = s->view->sc.ll.x; u < s->view->sc.ur.x; u += 1) {
      r = ray_unit(ray_transform(ray_view(u, v), mclip));
      c = RAY_SHADE_FUNC(RAY_SHADE_PARAMS);
      img_putc(s->img, u, v, col_dpymap(c));
    }
  }
  img_write(s->img,"stdout",0);
  exit(0);
}

Ray ray_view(int u, int v)
{
  Vector4 w = v4_m4mult(v4_make(u, v,s->view->sc.ur.z, 1), mdpy);
  return ray_make(v3_v4conv(v4_m4mult(v4_make(0, 0, 1, 0), mdpy)),
		  v3_make(w.x, w.y, w.z));
}

void init_render(Scene *s)
{
  mclip = m4_m4prod(s->view->Vinv, s->view->Cinv);
  mdpy = m4_m4prod(s->view->Pinv, s->view->Sinv);

  rc_sset(rc = NEWSTRUCT(RContext), s->view, s->lights, s->img);
}

void init_lang(void)
{
  lang_defun("scene", scene_parse);
  lang_defun("view", view_parse);
  lang_defun("dist_light", distlight_parse);
  lang_defun("plastic", plastic_parse);
  lang_defun("primobj", obj_parse);
  lang_defun("sphere", sphere_parse);
  lang_defun("polyobj", obj_parse);
  lang_defun("trilist", htrilist_parse);

  lang_defun("csgobj", obj_parse);
  lang_defun("csg_union", csg_union_parse);
  lang_defun("csg_inter", csg_inter_parse);
  lang_defun("csg_diff", csg_diff_parse);
  lang_defun("csg_prim", csg_prim_parse);

  lang_defun("group", group_parse);
  lang_defun("affine", xaffine_parse);

#ifdef RAY_MAP
  lang_defun("slideproj", slideproj_parse);
  lang_defun("shiny", shiny_parse);
  lang_defun("rough", rough_parse);
  lang_defun("textured", textured_parse);
  lang_defun("imagemap", imagemap_parse);
  lang_defun("chequer", chequer_parse);
#endif

  lang_defun("arg", arg_parse);
  lang_defun("motor", motor_parse);

  lang_defun("include", include_parse);
}
